#!/bin/csh -f
# MacMetaMod.csh MRH Nov 06
# Modify meta data in image file EXIF data
# Runs on command line or via Platypus droplet
# Dependent on exifmod.csh and exiftool package
# exif_progress_filter and cocoadialog for Platypus
# This file is duplicated in Platypus Resource dir
# for completeness (main code is encrypted).
# Note that Platypus seems to execute .cshrc 
# regardless of -f flag above.
# =================================================
# =================================================

# ==========================
# Set constants and defaults
# ==========================

set echo

onintr exit

set prog_name = "MacMetaMod.csh";
set version = "v1.8.2";
set credit = "Mark Harris Jan 2007"

set meter = "1" # Enable progress meter

set overwrite_string = "";
if (`uname -s` == 'Darwin') set overwrite_string = "-overwrite ";

set platypus_flag = 0
set prompt_string = "Enter keywords separated by commas : "
set append = " -append";
set single = "0"
set bubble_flag = 0;
set author = "";
set city = "";
set keywords = "";
set title = "";
set location = "";
set extra_msg = "";
set hard_key = ""
set hard_loc = ""
set hard_tag = ""
set hard_tit = ""

# Check if Platypus droplet, then if prog name encodes extras
# ===========================================================

set prog = `echo $prog_name | sed -e s/"\.[^\.]*"//`
if ($#argv > 0) then
if (`echo $argv[1] | grep -i $prog | wc -l`) then # first arg contains prog name
    set platypus_flag = 1
    set prog_extras = `echo $argv[1] | sed s/".*$prog"// | sed -e s/'.app$'// -e s/'^[^-]*-'/-/`
    if ("$prog_extras" != "") then # Something appended to prog name
     if (`echo $prog_extras | grep -i "\-key" | wc -l`) then # -key found
	set prm = 'key'
	set hard_key = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set keywords = "$hard_key"
	set hard_tag = "set"
     endif
     if (`echo $prog_extras | grep -i "\-loc" | wc -l`) then # -loc found
	set prm = 'loc'
	set hard_loc = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set location = "$hard_loc"
	set hard_tag = "set"
     endif
     if (`echo $prog_extras | grep -i "\-tit" | wc -l`) then # -tit found
	set prm = 'tit'
	set hard_tit = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set title = "$hard_tit"
	set hard_tag = "set"
     endif
    endif
endif
endif

# See if other programs are bundled
# =================================

if ("$platypus_flag" == "1") then
    set bundle = "0"

    if (-f "$1/Contents/Resources/exifmod.csh") set bundle = "$1";

    if ("$bundle" != "0") then
	# Note bundle flag must be first on line
	set exifmod = "$1/Contents/Resources/exifmod.csh -bundle $bundle";
	set exiftool = "$1/Contents/Resources/Image-ExifTool-stripped/exiftool";
	set progress_filter = "$1/Contents/Resources/exif_progress_filter.csh"
	set ccd = "$1/Contents/Resources/CocoaDialog.app/Contents/MacOS/CocoaDialog"
	set icon = "$1/Contents/Resources/appIcon.icns"
	set prefs_file = "$1/Contents/Resources/mmm_preferences.txt"
	set setprefs = "$1/Contents/Resources/set_preferences.csh"
    endif

    shift # Platypus offset. Important !

else

    set exifmod = "/Users/markh/ut/exifmod.csh";
    set progress_filter = "/Users/markh/ut/exif_progress_filter.csh"
    set exiftool = "/Applications/Image-ExifTool/exiftool";
    set icon = "/Users/markh/Desktop/macmetamod.app/Contents/Resources/appIcon.icns"
    set ccd = "/Applications/CocoaDialog.app/Contents/MacOS/CocoaDialog"
    set prefs_file = "/Users/markh/ut/mmm_preferences.txt";
    set setprefs = "/Users/markh/ut/set_preferences.csh";

endif

# If no files, show usage, offer prefs
# ====================================

if ($#argv == 0) then
  if ("$platypus_flag" == "1") then
    set ret = `"$ccd" msgbox --button1 "OK" --button2 "Preferences" --text "MacMetaMod $version" --informative-text "Drop files or directories on icon to add or replace keywords" --title "$credit"`
  if ("$ret[1]" == "2") then
    $setprefs $bundle $prefs_file
  endif
  else
    echo "Usage : $prog_name filename [filename] [directory]";
  endif
  exit
endif

# Get prefs from file if found
# ============================

if (-r "$prefs_file") then
    set author = `cat $prefs_file | grep "Author::" | sed -e s/"Author::"//`
    set city = `cat $prefs_file | grep "City::" | sed -e s/"City::"//`
endif

if ("$hard_tag" != "") goto doit # Skip prompting if something is hardcoded

# If only one file, show current keywords
# =======================================

set nonomatch

set curr_keys = ""
set curr_title = ""
set curr_author = ""
set curr_city = ""

if ($#argv == 1) then
  if (! -d "$argv[1]") then
    set single = "1";
    echo `date`
    set exifo = `$exiftool "$argv[1]" -keywords -title -creatorcontactinfociadrcity -creator `
    set exifo = `echo $exifo | sed -e s/"Creator City"/CreatorCity/ ` # Remove space to avoid match with Creator

    if (`echo $exifo | grep Keywords` != "") then
	set curr_keys = `echo $exifo | sed -e s/"Creator.*"// -e s/"CreatorCity.*"// -e s/"Title.*"// `
    endif

    if (`echo $exifo | grep Title` != "") set curr_title = `echo $exifo | sed -e s/"Creator.*"// -e s/"^.*Title"/\1/`

    if (`echo $exifo | grep "Creator "` != "") set curr_author = `echo $exifo | sed -e s/"^.*Creator "/\1/`
    if (`echo $exifo | grep CreatorCity` != "") set curr_city = `echo $exifo | sed -e s/"^.*CreatorCity"// | sed -e s/"Creator.*"//`

    #echo "ca $curr_author";
    ##echo "ck $curr_keys";
    ##echo "ct $curr_title";
    #echo "cc $curr_city";

    set curr_keys = "`echo $curr_keys | sed s/"^[^:]*:"//`"
    set curr_title = "`echo $curr_title | sed s/"^[^:]*:"//`"
    set curr_author = "`echo $curr_author | sed s/"^[^:]*:"//`"
    set curr_city = "`echo $curr_city | sed s/"^[^:]*:"//`"
    set curr_city_string = ""
    if ("$curr_city" != "") set curr_city_string = ", $curr_city"

    echo `date`
    echo "Current keys = $curr_keys"
    echo "Current title = $curr_title"
    echo "Current author = $curr_author"
    echo "Current city = $curr_city"

  endif
endif

# =================
# Prompt with cocoa

  if ("$platypus_flag" == "1") then

    if ($single == 1) then
	set extra_msg = "Nothing";
	if ("$author" != "") 	set extra_msg = "$author";
	if ("$city" != "") 	set extra_msg = "$extra_msg, $city";

	"$ccd" bubble --titles "Author :" "Title : " "Keywords :" "To be added from prefs :" --texts  "$curr_author$curr_city_string" "$curr_title" "$curr_keys" "$extra_msg" --background-tops "FFCC33" "FFCC33" "FFCC33" "FFCC33" --background-bottoms "FFDD66" "FFDD66" "FFDD66" "FFDD66"  --text-colors "0000ff" "0000ff" "0000ff" "000088" --icon-file "$icon" --no-timeout & 
	set bubpid = $!
	@ bubble_flag ++
	sleep 1 # So that subprocess has started before prompt comes up, so it gets focus
    endif

    set ret = `"$ccd" inputbox --title "$prog" --informative-text "$prompt_string" --button2 "Prompt for title" --button1 "Append" --button3 "Cancel" --float`
  else
    echo -n "Keywords to append : "
    set ret = "2$<"
    #echo "RET $ret"
  endif
  ##echo $ret > jjj.jjj 

  set retcode = `echo $ret | cut -c1`

  set keys = `echo $ret | cut -c2-`
  set keywords = "$keys"
  set append = " -append"

  if ("$retcode" == "3") then # Cancel
    goto exit
  endif

# Second prompt box

  if ("$retcode" == "2") then # Append and get title (second prompt box)
    if ("$platypus_flag" == "1") then
      set prompt_string = "Enter title for image : "
      set def_text = "$curr_title";
      if ("$single" != "1") set def_text = "Multiple files chosen, title will be set in all";

      set ret = `"$ccd" inputbox --title "$prog" --informative-text "$prompt_string" --text "$def_text" --button3 "Cancel" --button2 "Replace keywords and title" --button1 "Append keywords, replace title" --float`
    else
      echo -n "Title : "
      set ret = "1$<"
    endif
    set retcode = `echo $ret | cut -c1`
    if ("$retcode" == "3") then # Cancel
      goto exit
    endif

    set title = `echo $ret | cut -c2-`
    if (`echo $title | grep -i "Multiple files chosen"` != "") set title = "";

    if ("$retcode" == "2") then # Replace
      set append = ""
    endif

  endif

# =============================================================== Progress meter
doit:

#set echo

    if ("$single" != "1") then # Count files

	set filecount = 0
	set iopt=0
	argloop:
	@ iopt ++
	set arg = "$argv[$iopt]"
	if (-d "$arg") then
	  foreach file ("$arg"/*)
	  @ filecount ++
        end

	else
	  @ filecount ++
	endif
        if ($iopt < $#argv) goto argloop
    
        echo "File count : $filecount";

    else 
	set filecount = 1;
    endif

    set progress_text = "$filecount files to be processed";
    if ("$hard_key" != "") set progress_text = "$progress_text. keys = $keywords"
    if ("$hard_loc" != "") set progress_text = "$progress_text. loc = $location"
    if ("$hard_tit" != "") set progress_text = "$progress_text. tit = $title"

    if (("$meter" == "0" || "$single" == "1") || (! -f "$progress_filter") ) set meter = 0;

    if ("$meter" != "1" && "$single" != "1") then
	"$ccd" bubble --title "MacMetaMod" --text "$progress_text" --background-top "FFCC33" --background-bottom "FFDD66"  --text-color "0000ff" -icon-file "$icon" --no-timeout & 
	@ bubble_flag ++
	set bubpid = $!
    endif

# ===============================================================
# Modify each file in turn

set filelist = ""
set iopt=0
argloop2:
  @ iopt ++
  set arg = "$argv[$iopt]"
  set filelist = "$filelist "\""$arg"\"
if ($iopt < $#argv) goto argloop2

# Build up exifmod string

set exifmod = "$exifmod $overwrite_string"
if ("$author" != "") set exifmod = "$exifmod -author "\""$author"\"
if ("$keywords" != "") set exifmod = "$exifmod -keywords "\""$keywords"\""$append"
if ("$title" != "") set exifmod = "$exifmod -title "\""$title"\"
if ("$location" != "") set exifmod = "$exifmod -location "\""$location"\"
if ("$city" != "") set exifmod = "$exifmod -city "\""$city"\"

if ("$meter" == "0") then # Don't show meter (ie single file)

    eval $exifmod $filelist

else # Multiple files

     eval $exifmod -q -v0 $filelist |& "$progress_filter" $filecount |  "$ccd" progressbar --text "$progress_text" --percent 0 --title "$prog"

endif

# =========================================

exit:

# Tidy up (Kill lots of procs in case .cshrc started something)

if ($bubble_flag != 0) then
   kill -9 $bubpid
endif

exit
